@@ -80,6 +80,8 @@ static bool analyze_program(const char* name, CompileJob& job)
         job.setLanguage (CompileJob::Lang_CXX);
     else if (suffix == "cc")
         job.setLanguage (CompileJob::Lang_C);
+    else if (compiler_name == "clang")
+        job.setLanguage (CompileJob::Lang_C);
     else {
         job.setLanguage( CompileJob::Lang_Custom );
         job.setCompilerName( name ); // keep path
@@ -90,7 +92,7 @@ static bool analyze_program(const char* name, CompileJob& job)
 }
 
 bool analyse_argv( const char * const *argv,
-                   CompileJob &job, bool icerun )
+                   CompileJob &job, bool icerun, list<string> *extrafiles )
 {
     ArgumentsList args;
     string ofile;
@@ -108,6 +110,9 @@ bool analyse_argv( const char * const *argv,
     bool seen_s = false;
     bool seen_mf = false;
     bool seen_md = false;
+    bool fno_color_diagnostics = false;
+    // if rewriting includes and precompiling on remote machine, then cpp args are not local
+    Argument_Type Arg_Cpp = compiler_only_rewrite_includes( job ) ? Arg_Rest : Arg_Local;
     if( icerun ) {
         always_local = true;
         job.setLanguage( CompileJob::Lang_Custom );
@@ -186,6 +191,21 @@ bool analyse_argv( const char * const *argv,
 		    if (!*pos)
 		        break;
 		}
+                /* Some weird build systems pass directly additional assembler files.
+                 * Example: -Wa,src/code16gcc.s
+                 * Need to handle it locally then. Search if the first part after -Wa, does not start with -
+                 */
+                pos = a+3;
+                while (*pos) {
+                    if (*pos == ',' || *pos == ' ') {
+			pos++;
+                        continue;
+		    }
+                    if (*pos == '-')
+                        break;
+                    local = true;
+                    break;
+                }
                 if (local) {
                     always_local = true;
                     args.append(a, Arg_Local);
@@ -241,18 +261,34 @@ bool analyse_argv( const char * const *argv,
             } else if (str_equal("-include", a)) {
                 /* This has a duplicate meaning. it can either include a file
                    for preprocessing or a precompiled header. decide which one.  */
+
                 if (argv[i+1]) {
                     ++i;
                     std::string p = argv[i];
-                    if (access(p.c_str(), R_OK) && access((p + ".gch").c_str(), R_OK))
-                        always_local = true;  /* can't decide which one, let the compiler figure it
-                                                 out.  */
+                    string::size_type dot_index = p.find_last_of('.');
+
+                    if (dot_index != string::npos) {
+                        string ext = p.substr(dot_index + 1);
+
+                        if (ext[0] != 'h' && ext[0] != 'H' && access(p.c_str(), R_OK)
+                                && access((p + ".gch").c_str(), R_OK))
+                            always_local = true;
+
+                    } else
+                        always_local = true; /* Included file is not header.suffix or header.suffix.gch! */
+
                     args.append(a, Arg_Local);
                     args.append(argv[i], Arg_Local);
                 }
             } else if (str_equal("-D", a)
-                       || str_equal("-I", a)
-                       || str_equal("-U", a)
+                       || str_equal("-U", a) ) {
+                args.append(a, Arg_Cpp);
+                /* skip next word, being option argument */
+                if (argv[i+1]) {
+		    ++i;
+                    args.append(  argv[i], Arg_Cpp );
+		}
+            } else if (str_equal("-I", a)
                        || str_equal("-L", a)
                        || str_equal("-l", a)
                        || str_equal("-MF", a)
@@ -278,21 +314,68 @@ bool analyse_argv( const char * const *argv,
 		}
             } else if (str_startswith("-Wp,", a)
                  || str_startswith("-D", a)
-                 || str_startswith("-U", a)
-                 || str_startswith("-I", a)
+                 || str_startswith("-U", a)) {
+                args.append(a, Arg_Cpp);
+            } else if (str_startswith("-I", a)
                  || str_startswith("-l", a)
                  || str_startswith("-L", a)) {
                 args.append(a, Arg_Local);
-            } else if (str_equal("-undef", a)
-                 || str_equal("-nostdinc", a)
+            } else if (str_equal("-undef", a)) {
+                args.append(a, Arg_Cpp);
+            } else if (str_equal("-nostdinc", a)
                  || str_equal("-nostdinc++", a)
                  || str_equal("-MD", a)
                  || str_equal("-MMD", a)
                  || str_equal("-MG", a)
                  || str_equal("-MP", a)) {
                 args.append(a, Arg_Local);
+            } else if (str_equal("-fno-color-diagnostics", a)) {
+                fno_color_diagnostics = true;
+                args.append( a, Arg_Rest );
+            } else if (str_equal("-fcolor-diagnostics", a)) {
+                fno_color_diagnostics = false;
+                args.append( a, Arg_Rest );
+            } else if ( str_equal( "-flto", a ) ) {
+                // pointless when preprocessing, and Clang would emit a warning
+                args.append( a, Arg_Remote );
+            } else if (str_startswith("-fplugin=", a)) {
+                string file = a+strlen("-fplugin=");
+                if (access(file.c_str(), R_OK) == 0) {
+                    file = get_absfilename(file);
+                    extrafiles->push_back(file);
+                } else {
+                    always_local = true;
+                }
+                args.append("-fplugin="+file, Arg_Rest);
+            } else if (str_equal("-Xclang", a)) {
+                string a = argv[i];
+                if (argv[i+1]) {
+                    ++i;
+                    const char *p = argv[i];
+                    if (str_equal("-load", p)) {
+                        if (argv[i+1] && argv[i+2] && str_equal(argv[i+1], "-Xclang")) {
+                            args.append(a, Arg_Rest);
+                            args.append(p, Arg_Rest);
+                            string file = argv[i+2];
+                            if (access(file.c_str(), R_OK) == 0) {
+                                file = get_absfilename(file);
+                                extrafiles->push_back(file);
+                            } else {
+                                always_local = true;
+                            }
+                            args.append(argv[i+1], Arg_Rest);
+                            args.append(file, Arg_Rest);
+                            i += 2;
+                        }
+                    } else {
+                        args.append(a, Arg_Rest);
+                        args.append(p, Arg_Rest);
+                    }
+                }
             } else
                 args.append( a, Arg_Rest );
+        } else if (a[0] == '@') {
+            args.append(a, Arg_Local);
         } else {
             args.append( a, Arg_Rest );
         }
@@ -322,7 +405,10 @@ bool analyse_argv( const char * const *argv,
                 always_local = true;
                 break;
             }
-            if ( it->second != Arg_Rest || it->first.at( 0 ) == '-' )
+            if ( it->first == "-Xclang" ) {
+                ++it;
+                ++it;
+            } else if ( it->second != Arg_Rest || it->first.at( 0 ) == '-' || it->first.at( 0 ) == '@' )
                 ++it;
             else if ( ifile.empty() ) {
 #if CLIENT_DEBUG
@@ -400,6 +486,10 @@ bool analyse_argv( const char * const *argv,
     if ( ofile.empty() || (!stat( ofile.c_str(), &st ) && !S_ISREG( st.st_mode )))
         always_local = true;
 
+    // redirecting Clang's output will turn off its automatic coloring, so force it, unless disabled
+    if (compiler_is_clang(job) && colorify_possible() && !fno_color_diagnostics)
+        args.append("-fcolor-diagnostics", Arg_Rest);
+
     job.setFlags( args );
     job.setOutputFile( ofile );
 
