@@ -160,7 +160,7 @@ static void handle_user_break( int sig )
     user_break_signal = sig;
 
     if (child_pid != 0)
-      kill ( sig, child_pid);
+      kill ( child_pid, sig );
 
     signal( sig, handle_user_break );
 }
@@ -268,7 +268,7 @@ int build_local(CompileJob& job, MsgChannel *local_daemon, struct rusage *used)
         while( wait4( child_pid, &status, 0, used ) < 0 && errno == EINTR)
             ;
 
-        status = WEXITSTATUS(status);
+        status = shell_exit_status(status);
 
         signal( SIGINT, old_sigint );
         signal( SIGTERM, old_sigterm );
