@@ -69,7 +69,8 @@ static void dcc_show_usage(void)
 {
     printf(
 "Usage:\n"
-"   icecc [compile options] -o OBJECT -c SOURCE\n"
+"   icecc [compiler] [compile options] -o OBJECT -c SOURCE\n"
+"   icecc --build-native [compilertype] [file...]\n"
 "   icecc --help\n"
 "\n"
 "Options:\n"
@@ -77,17 +78,21 @@ static void dcc_show_usage(void)
 "   --version                  show version and exit\n"
 "   --build-native             create icecc environment\n"
 "Environment Variables:\n"
-"   ICECC                      if set to \"no\", just exec the real gcc\n"
-"   ICECC_VERSION              use a specific icecc environment, see create-env\n"
+"   ICECC                      if set to \"no\", just exec the real compiler\n"
+"   ICECC_VERSION              use a specific icecc environment, see icecc-create-env\n"
 "   ICECC_DEBUG                [info | warnings | debug]\n"
 "                              sets verboseness of icecream client.\n"
 "   ICECC_LOGFILE              if set, additional debug information is logged to the specified file\n"
 "   ICECC_REPEAT_RATE          the number of jobs out of 1000 that should be\n"
 "                              compiled on multiple hosts to ensure that they're\n"
-"                              producing the same output.  The default is 10.\n"
+"                              producing the same output.  The default is 0.\n"
 "   ICECC_PREFERRED_HOST       overrides scheduler decisions if set.\n"
-"   IECCC_CC                   set C compiler name (default gcc).\n"
+"   ICECC_CC                   set C compiler name (default gcc).\n"
 "   ICECC_CXX                  set C++ compiler name (default g++).\n"
+"   ICECC_CLANG_REMOTE_CPP     set to 1 or 0 to override remote precompiling with clang\n"
+"                              (requires clang -frewrite-includes option).\n"
+"   ICECC_IGNORE_UNVERIFIED    if set, hosts where environment cannot be verified are not used.\n"
+"   ICECC_EXTRAFILES           additional files used in the compilation.\n"
 "\n");
 }
 
@@ -95,14 +100,14 @@ static void icerun_show_usage(void)
 {
     printf(
 "Usage:\n"
-"   icerun [compile options] -o OBJECT -c SOURCE\n"
+"   icerun [command]\n"
 "   icerun --help\n"
 "\n"
 "Options:\n"
 "   --help                     explain usage and exit\n"
 "   --version                  show version and exit\n"
 "Environment Variables:\n"
-"   ICECC                      if set to \"no\", just exec the real gcc\n"
+"   ICECC                      if set to \"no\", just exec the real command\n"
 "   ICECC_DEBUG                [info | warnings | debug]\n"
 "                              sets verboseness of icecream client.\n"
 "   ICECC_LOGFILE              if set, additional debug information is logged to the specified file\n"
@@ -142,9 +147,9 @@ static string read_output( const char *command )
         log_error() << "no pipe " << strerror( errno ) << endl;
         return output;
     }
-    char buffer[PATH_MAX];
+    char buffer[1024];
     while ( !feof( f ) ) {
-        size_t bytes = fread( buffer, 1, PATH_MAX - 1, f );
+        size_t bytes = fread( buffer, 1, sizeof(buffer) - 1, f );
         buffer[bytes] = 0;
         output += buffer;
     }
@@ -154,37 +159,71 @@ static string read_output( const char *command )
     return output.substr(0, output.length()-1);
 }
 
-static int create_native()
+static int create_native( char** args )
 {
+    // args are [compiler] [files...]
+    const char *compiler = args[ 0 ];
+    char **extrafiles;
+    if( args[ 0 ] )
+        extrafiles = args + 1;
+    else // there were no arguments, set to null list
+        extrafiles = args;
+    int extracount = 0;
+    while ( extrafiles[ extracount ] )
+        ++extracount;
+    char **argv = new char*[4+extracount*2];
+    int pos = 0;
     struct stat st;
-    string gcc, gpp;
-
-    // perhaps we're on gentoo
-    if ( !lstat("/usr/bin/gcc-config", &st) ) {
-        string gccpath=read_output("/usr/bin/gcc-config -B") + "/";
-        gcc=gccpath + "gcc";
-        gpp=gccpath + "g++";
-    } else {
-        gcc = find_compiler( CompileJob::Lang_C );
-        gpp = find_compiler( CompileJob::Lang_CXX );
-    }
-
-    if ( gcc.empty() || gpp.empty())
-	return 1;
-
     if ( lstat( PLIBDIR "/icecc-create-env", &st ) ) {
 	log_error() << PLIBDIR "/icecc-create-env does not exist\n";
         return 1;
     }
+    argv[pos++] = strdup( PLIBDIR "/icecc-create-env"  );
 
-    char **argv = new char*[4];
-    argv[0] = strdup( PLIBDIR "/icecc-create-env"  );
-    argv[1] = strdup( gcc.c_str() );
-    argv[2] = strdup( gpp.c_str() );
-    argv[3] = NULL;
+    if ( compiler && strcmp( compiler, "clang" ) == 0 ) {
+        string clang = compiler_path_lookup( "clang" );
+        if ( clang.empty()) {
+            log_error() << "clang compiler not found\n";
+	    return 1;
+	}
+        if ( lstat( PLIBDIR "/compilerwrapper", &st ) ) {
+            log_error() << PLIBDIR "/compilerwrapper does not exist\n";
+            return 1;
+        }
+        argv[pos++] = strdup( "--clang" );
+        argv[pos++] = strdup( clang.c_str() );
+        argv[pos++] = strdup( PLIBDIR "/compilerwrapper"  );
+    } else { // "gcc" (default)
+        string gcc, gpp;
 
+        // perhaps we're on gentoo
+        if ( !lstat("/usr/bin/gcc-config", &st) ) {
+            string gccpath=read_output("/usr/bin/gcc-config -B") + "/";
+            gcc=gccpath + "gcc";
+            gpp=gccpath + "g++";
+        } else {
+            gcc = compiler_path_lookup( "gcc" );
+            gpp = compiler_path_lookup( "g++" );
+        }
+        // both C and C++ compiler are required
+        if ( gcc.empty())
+            gpp.clear();
+        if ( gpp.empty())
+            gcc.clear();
+        if ( gcc.empty() && gpp.empty()) {
+            log_error() << "gcc compiler not found\n";
+	    return 1;
+	}
+        argv[pos++] = strdup( "--gcc" );
+        argv[pos++] = strdup( gcc.c_str() );
+        argv[pos++] = strdup( gpp.c_str() );
+    }
+    for ( extracount = 0; extrafiles[ extracount ]; ++extracount ) {
+        argv[pos++] = strdup( "--addfile" );
+        argv[pos++] = strdup( extrafiles[ extracount ] );
+    }
+    argv[pos++] = NULL;
     return execv(argv[0], argv);
-
 }
 
 int main(int argc, char **argv)
@@ -223,8 +262,8 @@ int main(int argc, char **argv)
                 return 0;
             }
 	    if ( arg == "--build-native" )
-                return create_native();
-            if ( arg.size() > 0 && arg.at(0) == '/' )
+                return create_native(argv + 2);
+            if ( arg.size() > 0 )
                 job.setCompilerName(arg);
         }
     } else if ( find_basename( compiler_name ) == "icerun") {
@@ -243,10 +282,9 @@ int main(int argc, char **argv)
                 job.setCompilerName(arg);
         }
     } else {
-        char buf[ PATH_MAX ];
-        buf[ PATH_MAX - 1 ] = '\0';
+        std::string resolved;
         // check if it's a symlink to icerun
-        if( readlink( compiler_name.c_str(), buf, PATH_MAX - 1 ) >= 0 && find_basename( buf ) == "icerun" ) {
+        if( resolve_link( compiler_name, resolved ) == 0 && find_basename( resolved ) == "icerun" ) {
             icerun = true;
         }
     }
@@ -262,14 +300,50 @@ int main(int argc, char **argv)
      * see the EPIPE. */
     dcc_ignore_sigpipe(1);
 
-    local |= analyse_argv( argv, job, icerun );
+    list<string> extrafiles;
+    local |= analyse_argv( argv, job, icerun, &extrafiles );
 
     /* if ICECC is set to no, then run job locally */
     char* icecc = getenv("ICECC");
     if ( icecc && !strcasecmp(icecc, "no") )
         return build_local( job, 0 );
 
-    MsgChannel *local_daemon = Service::createChannel( "127.0.0.1", 10245, 0/*timeout*/);
+    if (const char *extrafilesenv = getenv("ICECC_EXTRAFILES")) {
+        for(;;) {
+            const char *colon = strchr(extrafilesenv, ':');
+            string file;
+            if (colon == NULL)
+                file = extrafilesenv;
+            else
+                file = string( extrafilesenv, colon - extrafilesenv );
+            struct stat st;
+            if (stat( file.c_str(), &st) == 0)
+                extrafiles.push_back( file );
+            else {
+                log_warning() << "File in ICECC_EXTRAFILES not found: " << file << endl;
+                return build_local( job, 0 );
+            }
+            if (colon == NULL)
+                break;
+            extrafilesenv = colon + 1;
+        }
+    }
+
+    /* try several options to reach the local daemon - 3 sockets, one TCP */
+    MsgChannel *local_daemon = Service::createChannel( "/var/run/icecc/iceccd.socket" );
+
+    if (!local_daemon)
+        local_daemon = Service::createChannel( "/var/run/iceccd.socket" );
+
+    if (!local_daemon) {
+        string path = getenv("HOME");
+        path += "/.iceccd.socket";
+        local_daemon = Service::createChannel( path );
+    }
+
+    if (!local_daemon)
+        local_daemon = Service::createChannel( "127.0.0.1", 10245, 0/*timeout*/);
+
     if ( ! local_daemon ) {
         log_warning() << "no local daemon found\n";
         return build_local( job, 0 );
@@ -284,8 +358,11 @@ int main(int argc, char **argv)
             } catch ( int x ) {
                 // we just build locally
             }
+        } else if ( !extrafiles.empty() && !IS_PROTOCOL_32( local_daemon )) {
+            log_warning() << "Local daemon is too old to handle compiler plugins.\n";
+            local = true;
         } else {
-            if ( !local_daemon->send_msg( GetNativeEnvMsg() ) ) {
+            if ( !local_daemon->send_msg( GetNativeEnvMsg( compiler_is_clang( job ) ? "clang" : "gcc", extrafiles ) ) ) {
                 log_warning() << "failed to write get native environment\n";
 		goto do_local_error;
             }
