@@ -43,6 +43,7 @@
 #include <algorithm>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <vector>
 #ifdef HAVE_RSYNC
 #include <librsync.h>
 #endif
@@ -56,6 +57,18 @@
 #define O_LARGEFILE 0
 #endif
 
+namespace {
+
+struct CharBufferDeleter
+{
+  char *buf;
+  CharBufferDeleter(char *b) : buf(b) {}
+  ~CharBufferDeleter()
+  { free(buf); }
+};
+
+}
+
 using namespace std;
 
 std::string remote_daemon;
@@ -139,7 +152,7 @@ rip_out_paths( const Environments &envs, map<string, string> &version_map, map<s
 
     for ( Environments::const_iterator it = envs.begin(); it != envs.end(); ++it )
     {
-        for ( int i = 0; i < 3; i++ )
+        for ( int i = 0; i < 4; i++ )
             if ( endswith( it->second, suffs[i], versfile ) )
             {
                 versionfile_map[it->first] = it->second;
@@ -161,15 +174,11 @@ get_absfilename( const string &_file )
         return _file;
 
     if ( _file.at( 0 ) != '/' ) {
-        static char buffer[PATH_MAX];
+        static std::vector<char> buffer(1024);
 
-#ifdef HAVE_GETCWD
-        getcwd(buffer, sizeof( buffer ) );
-#else
-        getwd(buffer);
-#endif
-        buffer[PATH_MAX - 1] = 0;
-        file = string( buffer ) + '/' + _file;
+        while ( getcwd( &buffer[0], buffer.size() - 1 ) == 0 && errno == ERANGE )
+            buffer.resize( buffer.size() + 1024 );
+        file = string( &buffer[0] ) + '/' + _file;
     } else {
         file = _file;
     }
@@ -285,8 +294,8 @@ static void write_server_cpp(int cpp_fd, MsgChannel *cserver)
 }
 
 
-static int build_remote_int(CompileJob &job, UseCSMsg *usecs, const string &environment, const string &version_file,
-                            const char *preproc_file, bool output )
+static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_daemon, const string &environment,
+                            const string &version_file, const char *preproc_file, bool output )
 {
     string hostname = usecs->hostname;
     unsigned int port = usecs->port;
@@ -334,8 +343,32 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, const string &envi
             log_error() << "write of environment failed" << endl;
             throw( 8 );
         }
+
+        if ( IS_PROTOCOL_31( cserver )) {
+            VerifyEnvMsg verifymsg( job.targetPlatform(), job.environmentVersion() );
+            if ( !cserver->send_msg( verifymsg ) )
+                throw( 22 );
+            Msg *msg = cserver->get_msg(60);
+            if ( msg && msg->type == M_VERIFY_ENV_RESULT ) {
+                if( !static_cast<VerifyEnvResultMsg*>( msg )->ok ) {
+                    // The remote can't handle the environment at all (e.g. kernel too old),
+                    // mark it as never to be used again for this environment.
+                    log_info() << "Host " << hostname << " did not succesfully verify environment." << endl;
+                    BlacklistHostEnvMsg blacklist( job.targetPlatform(), job.environmentVersion(), hostname );
+                    local_daemon->send_msg( blacklist );
+                    throw( 24 );
+                } else
+                    trace() << "Verified host " << hostname << " for environment " << job.environmentVersion()
+                        << " (" << job.targetPlatform() << ")" << endl;
+            } else
+                throw( 25 );
+        }
     }
 
+    if( !IS_PROTOCOL_31( cserver ) && ignore_unverified()) {
+        log_warning() << "Host " << hostname << " cannot be verified." << endl;
+        throw( 26 );
+    }
 
     CompileFileMsg compile_file( &job );
     {
@@ -372,10 +405,10 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, const string &envi
         while(waitpid( cpp_pid, &status, 0) < 0 && errno == EINTR)
             ;
 
-        if ( status ) { // failure
+        if ( shell_exit_status(status) != 0 ) { // failure
             delete cserver;
             cserver = 0;
-            return WEXITSTATUS( status );
+            return shell_exit_status( status );
         }
     } else {
         int cpp_fd = open( preproc_file, O_RDONLY );
@@ -423,7 +456,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, const string &envi
     {
         write(STDOUT_FILENO, crmsg->out.c_str(), crmsg->out.size() );
 
-        if(colorify_wanted())
+        if(colorify_wanted(job))
             colorify_output(crmsg->err);
         else
             write(STDERR_FILENO, crmsg->err.c_str(), crmsg->err.size() );
@@ -584,8 +617,10 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
 
     // older compilers do not support the options we need to make it reproducable
 #if defined(__GNUC__) && ( ( (__GNUC__ == 3) && (__GNUC_MINOR__ >= 3) ) || (__GNUC__ >=4) )
-    if ( rand() % 1000 < permill)
-        torepeat = 3;
+    if (!compiler_is_clang(job)) {
+        if ( rand() % 1000 < permill)
+            torepeat = 3;
+    }
 #endif
 
     trace() << job.inputFile() << " compiled " << torepeat << " times on " << job.targetPlatform() << "\n";
@@ -609,7 +644,8 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         fake_filename += get_absfilename( job.inputFile() );
         GetCSMsg getcs (envs, fake_filename, job.language(), torepeat,
 			job.targetPlatform(), job.argumentFlags(),
-		        preferred_host ? preferred_host : string() );
+		        preferred_host ? preferred_host : string(),
+		        ignore_unverified());
         if (!local_daemon->send_msg (getcs)) {
             log_warning() << "asked for CS\n";
             throw( 24 );
@@ -618,7 +654,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         UseCSMsg *usecs = get_server( local_daemon );
 	int ret;
 	if (!maybe_build_local (local_daemon, usecs, job, ret))
-            ret = build_remote_int( job, usecs,
+            ret = build_remote_int( job, usecs, local_daemon,
 	    			    version_map[usecs->host_platform],
 				    versionfile_map[usecs->host_platform],
 				    0, true );
@@ -626,8 +662,9 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         return ret;
     } else
     {
-        char preproc[PATH_MAX];
-        dcc_make_tmpnam( "icecc", ".ix", preproc, 0 );
+        char *preproc = 0;
+        dcc_make_tmpnam( "icecc", ".ix", &preproc, 0 );
+        const CharBufferDeleter preproc_holder(preproc);
         int cpp_fd = open(preproc, O_WRONLY );
 	/* When call_cpp returns normally (for the parent) it will have closed
 	   the write fd, i.e. cpp_fd.  */
@@ -638,9 +675,9 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         }
         int status = 255;
         waitpid( cpp_pid, &status, 0);
-        if ( status ) { // failure
+        if ( shell_exit_status(status) ) { // failure
             ::unlink( preproc );
-            return WEXITSTATUS( status );
+            return shell_exit_status( status );
         }
 
         char rand_seed[400]; // "designed to be oversized" (Levi's)
@@ -648,7 +685,8 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         job.appendFlag( rand_seed, Arg_Remote );
 
         GetCSMsg getcs (envs, get_absfilename( job.inputFile() ), job.language(), torepeat,
-                job.targetPlatform(), job.argumentFlags(), preferred_host ? preferred_host : string() );
+                job.targetPlatform(), job.argumentFlags(), preferred_host ? preferred_host : string(),
+                ignore_unverified());
 
 
         if (!local_daemon->send_msg (getcs)) {
@@ -668,12 +706,13 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
 
         for ( int i = 0; i < torepeat; i++ ) {
             jobs[i] = job;
-            char buffer[PATH_MAX];
+            char *buffer = 0;
             if ( i ) {
-                dcc_make_tmpnam( "icecc", ".o", buffer, 0 );
+                dcc_make_tmpnam( "icecc", ".o", &buffer, 0 );
                 jobs[i].setOutputFile( buffer );
             } else
-                sprintf( buffer, "%s", job.outputFile().c_str() );
+                buffer = strdup( job.outputFile().c_str() );
+            const CharBufferDeleter buffer_holder( buffer );
 
             umsgs[i] = get_server( local_daemon );
             remote_daemon = umsgs[i]->hostname;
@@ -686,7 +725,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                 try {
 		    if (!maybe_build_local (local_daemon, umsgs[i], jobs[i], ret))
 			ret = build_remote_int(
-				  jobs[i], umsgs[i],
+				  jobs[i], umsgs[i], local_daemon,
 				  version_map[umsgs[i]->host_platform],
                                   versionfile_map[umsgs[i]->host_platform],
 				  preproc, i == 0 );
@@ -713,7 +752,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                     misc_error = true;
                     break;
                 }
-                exit_codes[jobmap[pid]] = WEXITSTATUS( status );
+                exit_codes[jobmap[pid]] = shell_exit_status( status );
             }
         }
 
