@@ -26,6 +26,7 @@
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <arpa/inet.h>
 #include <sys/select.h>
 #include <netinet/in.h>
@@ -44,6 +45,9 @@
 #include <assert.h>
 #include <minilzo.h>
 #include <stdio.h>
+#ifdef HAVE_LIBCAP_NG
+#include <cap-ng.h>
+#endif
 
 #include "logging.h"
 #include "job.h"
@@ -656,10 +660,35 @@ MsgChannel *Service::createChannel (const string &hostname, unsigned short p, in
       if (connect (remote_fd, (struct sockaddr *) &remote_addr, sizeof (remote_addr)) < 0)
         {
           close( remote_fd );
-	  trace() << "connect failed\n";
+	  trace() << "connect failed on " << hostname << endl;
           return 0;
         }
     }
+  trace() << "connected to " << hostname << endl;
+  return createChannel(remote_fd, (struct sockaddr *)&remote_addr, sizeof( remote_addr ));
+}
+
+MsgChannel *Service::createChannel (const string &socket_path)
+{
+  int remote_fd;
+  struct sockaddr_un remote_addr;
+
+  if ((remote_fd = socket (AF_UNIX, SOCK_STREAM, 0)) < 0)
+    {
+      log_perror("socket()");
+      return 0;
+    }
+
+  remote_addr.sun_family = AF_UNIX;
+  strncpy(remote_addr.sun_path, socket_path.c_str(), sizeof(remote_addr.sun_path) - 1);
+
+  if (connect (remote_fd, (struct sockaddr *) &remote_addr, sizeof (remote_addr)) < 0)
+    {
+      close( remote_fd );
+      trace() << "connect failed on " << socket_path << endl; 
+      return 0;
+    }
+  trace() << "connected to " << socket_path << endl;
   return createChannel(remote_fd, (struct sockaddr *)&remote_addr, sizeof( remote_addr ));
 }
 
@@ -940,6 +969,9 @@ MsgChannel::get_msg(int timeout)
     case M_GET_INTERNALS: m = new GetInternalStatus; break;
     case M_STATUS_TEXT: m = new StatusTextMsg; break;
     case M_CS_CONF: m = new ConfCSMsg; break;
+    case M_VERIFY_ENV: m = new VerifyEnvMsg; break;
+    case M_VERIFY_ENV_RESULT: m = new VerifyEnvResultMsg; break;
+    case M_BLACKLIST_HOST_ENV: m = new BlacklistHostEnvMsg; break;
     case M_TIMEOUT: break;
     }
   if (!m) {
@@ -1240,6 +1272,14 @@ GetCSMsg::fill_from_channel (MsgChannel *c)
   preferred_host = string();
   if (IS_PROTOCOL_22(c))
     *c >> preferred_host;
+  if (IS_PROTOCOL_31(c))
+    {
+      uint32_t ign;
+      *c >> ign;
+      ignore_unverified = ( ign != 0 );
+    }
+  else
+    ignore_unverified = false;
 }
 
 void
@@ -1255,6 +1295,8 @@ GetCSMsg::send_to_channel (MsgChannel *c) const
   *c << client_id;
   if (IS_PROTOCOL_22(c))
     *c << preferred_host;
+  if (IS_PROTOCOL_31(c))
+    *c << uint32_t( ignore_unverified );
 }
 
 void
@@ -1312,6 +1354,12 @@ CompileFileMsg::fill_from_channel (MsgChannel *c)
   string target;
   *c >> target;
   job->setTargetPlatform( target );
+  if (IS_PROTOCOL_30(c))
+  {
+    string compilerName;
+    *c >> compilerName;
+    job->setCompilerName( compilerName );
+  }
 }
 
 void
@@ -1320,10 +1368,35 @@ CompileFileMsg::send_to_channel (MsgChannel *c) const
   Msg::send_to_channel (c);
   *c << (uint32_t) job->language();
   *c << job->jobID();
-  *c << job->remoteFlags();
+  if (IS_PROTOCOL_30(c))
+    *c << job->remoteFlags();
+  else
+  {
+    if (job->compilerName().find("clang") != string::npos)
+    { // Hack for compilerwrapper.
+        std::list<std::string> flags = job->remoteFlags();
+        flags.push_front("clang");
+        *c << flags;
+    }
+    else
+      *c << job->remoteFlags();
+  }
   *c << job->restFlags();
   *c << job->environmentVersion();
   *c << job->targetPlatform();
+  if (IS_PROTOCOL_30(c))
+    *c << remote_compiler_name();
+}
+
+// Environments created by icecc-create-env always use the same binary name
+// for compilers, so even if local name was e.g. c++, remote needs to
+// be g++ (before protocol version 30 remote CS even had /usr/bin/{gcc|g++}
+// hardcoded).  For clang, the binary is just clang for both C/C++.
+string CompileFileMsg::remote_compiler_name() const
+{
+    if (job->compilerName().find("clang") != string::npos)
+        return "clang";
+    return job->language() == CompileJob::Lang_CXX ? "g++" : "gcc";
 }
 
 CompileJob *
@@ -1479,8 +1552,12 @@ JobDoneMsg::send_to_channel (MsgChannel *c) const
 LoginMsg::LoginMsg(unsigned int myport, const std::string &_nodename, const std::string _host_platform)
   : Msg(M_LOGIN), port( myport ), noremote( false ), chroot_possible( false ), nodename( _nodename ), host_platform( _host_platform )
 {
+#ifdef HAVE_LIBCAP_NG
+  chroot_possible = capng_have_capability( CAPNG_PERMITTED, CAP_SYS_CHROOT );
+#else
   // check if we're root
   chroot_possible = ( geteuid() == 0 );
+#endif
 }
 
 void
@@ -1553,6 +1630,26 @@ StatsMsg::send_to_channel (MsgChannel *c) const
   *c << freeMem;
 }
 
+void
+GetNativeEnvMsg::fill_from_channel (MsgChannel *c)
+{
+  Msg::fill_from_channel (c);
+  if (IS_PROTOCOL_32(c)) {
+    *c >> compiler;
+    *c >> extrafiles;
+  }
+}
+
+void
+GetNativeEnvMsg::send_to_channel (MsgChannel *c) const
+{
+  Msg::send_to_channel (c);
+  if (IS_PROTOCOL_32(c)) {
+    *c << compiler;
+    *c << extrafiles;
+  }
+}
+
 void
 UseNativeEnvMsg::fill_from_channel (MsgChannel *c)
 {
@@ -1693,6 +1790,56 @@ StatusTextMsg::send_to_channel (MsgChannel *c) const
   *c << text;
 }
 
+void
+VerifyEnvMsg::fill_from_channel (MsgChannel *c)
+{
+  Msg::fill_from_channel( c );
+  *c >> environment;
+  *c >> target;
+}
+
+void
+VerifyEnvMsg::send_to_channel (MsgChannel *c) const
+{
+  Msg::send_to_channel( c );
+  *c << environment;
+  *c << target;
+}
+
+void
+VerifyEnvResultMsg::fill_from_channel (MsgChannel *c)
+{
+  Msg::fill_from_channel( c );
+  uint32_t read_ok;
+  *c >> read_ok;
+  ok = read_ok != 0;
+}
+
+void
+VerifyEnvResultMsg::send_to_channel (MsgChannel *c) const
+{
+  Msg::send_to_channel( c );
+  *c << uint32_t( ok );
+}
+
+void
+BlacklistHostEnvMsg::fill_from_channel (MsgChannel *c)
+{
+  Msg::fill_from_channel( c );
+  *c >> environment;
+  *c >> target;
+  *c >> hostname;
+}
+
+void
+BlacklistHostEnvMsg::send_to_channel (MsgChannel *c) const
+{
+  Msg::send_to_channel( c );
+  *c << environment;
+  *c << target;
+  *c << hostname;
+}
+
 /*
 vim:cinoptions={.5s,g0,p5,t0,(0,^-0.5s,n-0.5s:tw=78:cindent:sw=4:
 */
