@@ -1160,6 +1160,14 @@ handle_login (CS *cs, Msg *_m)
           return false;
 
   dbg << "login " << m->nodename << " protocol version: " << cs->protocol;
+#if 1
+  dbg << " [";
+  for (Environments::const_iterator it = m->envs.begin();
+       it != m->envs.end(); ++it)
+    dbg << it->second << "(" << it->first << "), ";
+  dbg << "]" << endl;
+#endif
+
   handle_monitor_stats( cs );
 
   /* remove any other clients with the same IP, they must be stale */
@@ -1177,14 +1185,6 @@ handle_login (CS *cs, Msg *_m)
 
   css.push_back (cs);
 
-#if 1
-  dbg << " [";
-  for (Environments::const_iterator it = m->envs.begin();
-       it != m->envs.end(); ++it)
-    dbg << it->second << "(" << it->first << "), ";
-  dbg << "]" << endl;
-#endif
-
   /* Configure the daemon */
   if (IS_PROTOCOL_24( cs ))
     cs->send_msg (ConfCSMsg(icecream_bench_code));
@@ -1923,18 +1923,26 @@ main (int argc, char * argv[])
   gid_t user_gid;
   bool warn_icecc_user = false;
 
-  struct passwd *pw = getpwnam("icecc");
-  if (pw)
+  if ( getuid() == 0 )
     {
-      user_uid = pw->pw_uid;
-      user_gid = pw->pw_gid;
-     }
+    struct passwd *pw = getpwnam("icecc");
+    if (pw)
+      {
+        user_uid = pw->pw_uid;
+        user_gid = pw->pw_gid;
+      }
+    else
+      {
+        warn_icecc_user = true;
+        user_uid = 65534;
+        user_gid = 65533;
+      }
+    }
   else
     {
-      warn_icecc_user = true;
-      user_uid = 65534;
-      user_gid = 65533;
-     }
+      user_uid = getuid();
+      user_gid = getgid();
+    }
 
   while ( true ) {
     int option_index = 0;
@@ -1994,7 +2002,7 @@ main (int argc, char * argv[])
     case 'u':
         if ( optarg && *optarg )
           {
-            pw = getpwnam( optarg );
+            struct passwd *pw = getpwnam( optarg );
             if ( !pw )
               {
                 usage( "Error: -u requires a valid username" );
