@@ -26,7 +26,7 @@ extern "C" {
 
 int dcc_make_tmpnam(const char *prefix,
                     const char *suffix,
-                    char *name_ret, int relative);
+                    char **name_ret, int relative);
 
 #ifdef __cplusplus
 }
